// Contains class definitions
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
#include "pDisplay.h"
#include <strstream.h>
#include <iomanip.h>



//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//
// Global Variables
//
const char *time_modestr[]=LABELS_TIME_MODE_ICON;


//two pixmaps for the play state: one for playing, one for pausing
RawPixData rplayd={ 
  TRUE,
  1.0,
  1.0,
  16,
  1,
  TRUE,
  {".25,.25  .74,.49  .74,.5  .25,.74  .25,.25",
   "","","","","","","","",""},
  "5",
  "p" };

RawPixData rpaused={ 
  TRUE,
  1.0,
  1.0,
  16,
  2,
  TRUE,
  {".25,.25  .374,.25  .374,.74  .25,.74  .25,.25",
   ".5,.25  .624,.25  .624,.74  .5,.74  .5,.25",
   "","","","","","","",""},
  "5 5",
  "pp" };

RawPixData repeatd={ 
  TRUE,
  1.0,
  1.0,
  16,
  5,
  FALSE,
  {".2,.3  .8,.7",
   ".44,.16  .6,.3",".6,.3  .44,.43", 
   ".54,.57  .4,.7",".4,.7  .54,.84",
   "","","","",""},
  "2 2 2 2 2",
  "cllll" };

RawPixData rep1d={ 
  TRUE,
  1.0,
  1.0,
  16,
  7,
  FALSE,
  {".1,.3  .6,.7",
   ".4,.16  .25,.3", ".25,.3  .4,.43",
   ".25,.57  .4,.7", ".4,.7  .25,.84",
   ".85,.3  .85,.7", ".85,.3  .7,.4",
   "","",""},
  "2 2 2 2 2 2 2",
  "cllllll" };







// ***************
//  Here, we define the actual functions




//----------------------------------------------------------
//
// Inlined methods.
//
//----------------------------------------------------------




// Used by mousePressEvent() and mouseReleaseEvent() to check if the
// event is over a certain child widget
//
inline bool pDisplay::isInBox( QPoint posn, QPoint corner, QSize range )
{
  int xf,yf;

  xf = corner.x() + range.width();
  yf = corner.y() + range.height();
  return ( (corner.x() <= posn.x()) && (posn.x() <= xf) &&
		 (corner.y() <= posn.y()) && (posn.y() <= yf) );
}




//||||||||||||||||||||||||||||||||||||||||||||||||
//
//  Class pDisplay
//
//||||||||||||||||||||||||||||||||||||||||||||||||



// Constructor for the pDisplay object.
//
pDisplay::pDisplay( QWidget *parent, QColor& dispfgcol,
				QFont& ifn, float scale, bool compact ) 
   : QFrame( parent )
{
   QFont tmfont,infofont;
   char *sizestr;
   
   // Perform some initial settings.
   setEnabled( true );         //enabled event receiving.
   df_p = QPoint( 0, 0 );
   setFrameStyle( Panel | Sunken );
   setLineWidth( 2 );
   infofont = ifn;
   tmfont = QFont(SMALLFONT);
   
   if(compact)
	 {
	 sizestr = COMPACT_TITLE_STRING;
	 }
   else
	 {
	 sizestr = WIDE_TITLE_STRING;
	 }


   //Now create and position the children
   makePixmaps( scale );
   createChildren( sizestr );
   changeDTextFont(infofont);
   changeTDMFont(tmfont);
   sizeChildren( scale );
   locateChildren( compact );
   moveChildren();
   connectChildren();
   setChildForegroundColor( dispfgcol );
   
   //Lastly, set some default labels to activate the display.
   trackname->setText("...Track Title... ...Track Title... ...TrackTitle...");
   useTitleLabel();
   newPlayMode(psOff | pmNormal);
   volm->setLevel( 1.0 );
   moveAll( df_p );
}


// Constructor for the pDisplay object.
//
pDisplay::~pDisplay( void )
{
   delete track;
   delete time;
   delete titlename;
   delete trackname;
   delete titleboxlabel;
   delete timelabel;
   delete stateicon;
   delete modeicon;
   delete volm;
}


// Creates all of the child widgets in this frame
//
void pDisplay::createChildren( const char *boxwidthstr )
{
  track = new QeLCDNumber( this, "track" );
  track->setMode( QeLCDNumber::DEC );
  track->setSmallDecimalPoint( FALSE );
  track->setNumDigits( 2 );
  track->setFrameStyle( NoFrame );
  track->setSegmentStyle( QLCDNumber::Filled );
  track->display( NULL_TRACK );

  time = new QeLCDNumber( this, "time" );
  time->setMode( QeLCDNumber::DEC );
  time->setSmallDecimalPoint( FALSE );
  time->setNumDigits( 5 );
  time->setFrameStyle( NoFrame );
  time->setSegmentStyle( QLCDNumber::Filled );
  time->display( NULL_TIME );
  
  timelabel = new QeLabel( this, "timelabel" );
  timelabel->setText( time_modestr[3] );
  timelabel->setFrameStyle( NoFrame );
  timelabel->setAlignment( AlignTop | AlignRight );
  
  titleboxlabel = new QeLabel( this, "titleboxlabel" );
  titleboxlabel->setFrameStyle( Plain | Box );
  titleboxlabel->setLineWidth( 1 );
  useArtistLabel();
  titleboxlabel->setAlignment( AlignVCenter | AlignLeft );
  
  trackname = new QeLabel( this, "trackname" );
  trackname->setFrameStyle( Plain | Box );
  trackname->setLineWidth( 1 );
  trackname->setText( boxwidthstr );
  trackname->setAlignment( AlignVCenter | AlignLeft );
  
  titlename = new QeLabel( this, "titlename" );
  titlename->setFrameStyle( NoFrame );
  titlename->setText( "Title Name" );
  titlename->setAlignment( AlignVCenter | AlignLeft );
  
  stateicon = new QeLabel( this, "playstate_icon" );
  stateicon->setFrameStyle( NoFrame );
  stateicon->setAlignment( AlignVCenter | AlignLeft );

  modeicon = new QeLabel( this, "playmode_icon" );
  modeicon->setFrameStyle( NoFrame );
  modeicon->setAlignment( AlignVCenter | AlignLeft );

  volm = new QMeter( this, "volume_meter" );
  volm->setFrameStyle( NoFrame );
}


// Connects all of the child pressed/released signals to the
// corresponding slots.
//
void pDisplay::connectChildren(void)
{
  connect(track,SIGNAL(pressed(int,int)),
		SLOT(childPressed(int,int)) );
  connect(time,SIGNAL(pressed(int,int)),
		SLOT(childPressed(int,int)) );
  connect(timelabel,SIGNAL(pressed(int,int)),
		SLOT(childPressed(int,int)) );
  connect(titleboxlabel,SIGNAL(pressed(int,int)),
		SLOT(childPressed(int,int)) );
  connect(trackname,SIGNAL(pressed(int,int)),
		SLOT(childPressed(int,int)) );
  connect(titlename,SIGNAL(pressed(int,int)),
		SLOT(childPressed(int,int)) );
  connect(stateicon,SIGNAL(pressed(int,int)),
		SLOT(childPressed(int,int)) );
  //
  connect(track,SIGNAL(released(int,int)),
		SLOT(childReleased(int,int)) );
  connect(time,SIGNAL(released(int,int)),
		SLOT(childReleased(int,int)) );
  connect(timelabel,SIGNAL(released(int,int)),
		SLOT(childReleased(int,int)) );
  connect(titleboxlabel,SIGNAL(released(int,int)),
		SLOT(childReleased(int,int)) );
  connect(trackname,SIGNAL(released(int,int)),
		SLOT(childReleased(int,int)) );
  connect(titlename,SIGNAL(released(int,int)),
		SLOT(childReleased(int,int)) );
  connect(stateicon,SIGNAL(released(int,int)),
		SLOT(childReleased(int,int)) );
}


// Sets the size of all of the child widgets.
//
void pDisplay::makePixmaps( float scale )
{
   PixData *tmp;

   tmp = new PixData( rpaused, scale );
   pause_pd = (*tmp);
   delete tmp;
   tmp = new PixData( rplayd, scale );
   play_pd = (*tmp);
   delete tmp;
   tmp = new PixData( repeatd, scale );
   repeat_pd = (*tmp);
   delete tmp;
   tmp = new PixData( rep1d, scale );
   rep1_pd = (*tmp);
   delete tmp;

   pause_pm = pause_pd.toPixmap();
   play_pm = play_pd.toPixmap();
   rep_pm = repeat_pd.toPixmap();
   rep1_pm = rep1_pd.toPixmap();
   blank_pm = QPixmap(play_pm.size());
}


// Sets the size of all of the child widgets.
//
void pDisplay::sizeChildren( float scale )
{
  // Set some fixed sizes, mainly the LCD.
  track_s.setWidth( (int)(scale*36) );
  track_s.setHeight( (int)(scale*32) );
  time_s.setWidth( (int)(scale*84) );
  time_s.setHeight( (int)(scale*32) );
  bl_time_s = timelabel->sizeHint();
  bl_title_s = titleboxlabel->sizeHint();
  vol_s.setWidth( (int)(scale*12) );
  vol_s.setHeight( (int)(scale*32) );


  // These three have a fixed height and widths based on pre-existing
  // sizes.
  stateicon_s = play_pm.size();
  modeicon_s = rep1_pm.size();
  trackname_s = trackname->sizeHint();
  df_s.setWidth( trackname_s.width() + 2*bsep );
  title_s.setHeight( bl_title_s.height() );
  title_s.setWidth( df_s.width() - bl_title_s.width() - 3*bsep );

  // Now resize everything.
  track->resize( track_s );
  time->resize( time_s );
  timelabel->resize( bl_time_s );
  titleboxlabel->resize( bl_title_s );
  trackname->resize( trackname_s );
  titlename->resize( title_s );
  modeicon->resize( modeicon_s );
  stateicon->resize( stateicon_s );
  volm->resize( vol_s );
  // Need to also reinit volume meter after the resize.
  volm->initMeter();
  if( scale > SCALE_SIZE_CUTOFF )
	{
	volm->setLightSize( 0.05 );
	}
  else
	{
	volm->setLightSize( 0.1 );
	}
}


// Positions all of the child widgets
//
void pDisplay::locateChildren( bool compact )
{
  int sepfac;

  //Set some scales
  if(compact)
	{
	sepfac=1;
	}
  else
	{
	sepfac=2;
	}


  //Start positioning.
  bl_title_p = QPoint( bsep, bsep );
  title_p = bl_title_p + QPoint( bl_title_s.width() + bsep, 0 );
  trackname_p = bl_title_p + QPoint( 0, title_s.height() + bsep );
  time_p = trackname_p + QPoint( 0, trackname_s.height() + 2*bsep );

  //These all have the same y-coordinate as time_p
  vol_p = stateicon_p = modeicon_p 
    = track_p = bl_time_p = time_p;
  time_p.setX( df_s.width() - bsep - time_s.width() );
  bl_time_p.setX( time_p.x() - bl_time_s.width() );
  track_p.setX( bl_time_p.x() - sepfac*bsep - track_s.width() );
  stateicon_p.setX( track_p.x() - (sepfac-1)*bsep - stateicon_s.width() );
  modeicon_p.setX( stateicon_p.x() - (sepfac-1)*bsep 
			    - modeicon_s.width() );
  vol_p.setX( bl_title_p.x() );

  df_s.setHeight( time_p.y() + time_s.height() + bsep );

  // Lastly, resize the main widget.
  resize( df_s );
}


// Moves all of the children en-masse
//
void pDisplay::moveChildren( void )
{
  titleboxlabel->move( df_p + bl_title_p );
  titlename->move( df_p + title_p );
  trackname->move( df_p + trackname_p );
  time->move( df_p + time_p );
  timelabel->move( df_p + bl_time_p );
  track->move( df_p + track_p );
  modeicon->move( df_p + modeicon_p );
  stateicon->move( df_p + stateicon_p );
  volm->move( df_p + vol_p );
}


// Sets the foreground color of the widget
//
void pDisplay::setChildForegroundColor( QColor& fgc )
{
   childfg = fgc;
   resetChildPalette();
}


// Resets the color of the child palette and resets the palettes of
// all children
//
void pDisplay::resetChildPalette(void)
{
   QPalette childpal;
   QColor curbg;

   curbg = backgroundColor();
   childpal.setNormal(QColorGroup(childfg, 
						    curbg,
						    childfg.light(),
						    childfg.dark(150),
						    childfg.dark(),
						    childfg,
						    curbg)
				  );
   childpal.setActive( childpal.normal() );
   childpal.setDisabled( childpal.normal() );

   // Now recolor all of the other widgets
   track->setPalette( childpal );
   time->setPalette( childpal );
   timelabel->setPalette( childpal );
   titleboxlabel->setPalette( childpal );
   trackname->setPalette( childpal );
   titlename->setPalette( childpal );
   stateicon->setPalette( childpal );
   modeicon->setPalette( childpal );
   volm->setPalette( childpal );

   // Repaint the pixmaps.
   pause_pd.setColor( childfg, curbg );
   play_pd.setColor( childfg, curbg );
   repeat_pd.setColor( childfg, curbg );
   rep1_pd.setColor( childfg, curbg );
   blank_pm.fill(curbg);
   pause_pm = pause_pd.toPixmap();
   play_pm = play_pd.toPixmap();
   rep_pm = repeat_pd.toPixmap();
   rep1_pm = rep1_pd.toPixmap();

   // Lastly, redraw the QPixmap-based widgets.
   volm->Redraw();
   newPlayMode(oldpmode);
}


// This changes the palette for the entire widget.
//
void pDisplay::changePalette( QPalette& p )
{
   setPalette(p);
   resetChildPalette();
}


// These next two functions alter the two basic fonts used by
// pDisplay.  NOTE:  They do not resize anything!
void pDisplay::changeDTextFont( QFont& f )
{
   if(f.pointSize() > 20)
	 f.setPointSize(20);

   if(f.pointSize() < 8)
	 f.setPointSize(8);

   titleboxlabel->setFont(f);
   trackname->setFont(f);
   titlename->setFont(f);
}
//
void pDisplay::changeTDMFont( QFont& f )
{
   if(f.pointSize() > 8)
	 f.setPointSize(8);

   timelabel->setFont(f);
   modeicon->setFont(f);
}


// Reimplemented virtual function to receive a mouse pressed
// event. Sends out the pressed() signal.
//
void pDisplay::mousePressEvent( QMouseEvent *me )
{
  childPressed(me->pos());
}


// Reimplemented virtual function to receive a mouse released event.
// It sends out the released() signal.
//
void pDisplay::mouseReleaseEvent( QMouseEvent *me )
{
  childReleased(me->pos());
}


// Reimplemented virtual function to receive a mouse move event.  It
// sends out the moving() signal.
//
void pDisplay::mouseMoveEvent( QMouseEvent *me )
{
  emit moving( mapToGlobal(me->pos()) );
}


// Slot to change the track number display
//
void pDisplay::newTrack(int tn)
{
  char dispstr[4];
  strstream ds(dispstr,4);

  if(tn < 0)
	{
	track->display( NULL_TRACK );
	}
  else
	{
	ds << setw(2) << dec << tn << ends << flush;
	track->display( dispstr );
	}
}


// Slot to change the time display
//
void pDisplay::newTime(int tt)
{
  char dispstr[8];
  strstream ds(dispstr,8);

  if(tt < 0)
	{
	time->display( NULL_TIME );
	}
  else
	{
	ds << setfill(' ') << setw(2) << dec << tt/60 << ":" 
	   << setfill('0') << setw(2) << dec << tt%60
	   << ends << flush;
	time->display( dispstr );
	}
}


// Slot to change the play state and/or play mode icon
//
void pDisplay::newPlayMode(int mn)
{
  // Alter the playstate icon first
  switch(mn & PLAY_STATE_MASK)
	{
	  case psPlaying:
	  {
	  stateicon->setPixmap( play_pm );
	  break;
	  }
	  
	  case psPaused:
	  {
	  stateicon->setPixmap( pause_pm );
	  break;
	  }
	  
	  case psOff:
	  {
	  stateicon->setPixmap( blank_pm );
	  break;
	  }

	  default:
	  {
	  break;
	  }
	}

  // Now check the mode and change if necessary
  switch(mn & PLAY_MODE_MASK)
 	{
 	  case pmRepeatAll:
 	  {
	  modeicon->setPixmap( rep_pm );
 	  break;
 	  }
	  
 	  case pmRepeat1:
 	  {
	  modeicon->setPixmap( rep1_pm );
 	  break;
 	  }
	  
 	  case pmShuffle:
 	  {
	  modeicon->setText( "?" );
 	  break;
 	  }
	  
 	  case pmNormal:
 	  {
	  modeicon->setPixmap( blank_pm );
 	  break;
 	  }

	  default:
	  {
	  break;
	  }
 	}
  
  // Store the mode in case we need to call this method for redrawing
  // purposes.
  oldpmode = mn;
}


// Slot to change the time mode label
//
void pDisplay::newTimeMode(int mn)
{
  timelabel->setText(time_modestr[mn]);
}


// Slot to change the cd title name
//
void pDisplay::newTitleName(char *titlestr)
{
  titlename->setText(titlestr);
}


// Slot to change the cd title name
//
void pDisplay::newTrackName(char *trckstr)
{
  trackname->setText(trckstr);
}


// Slot to change the volume meter level
//
void pDisplay::newVolume(float newvol)
{
  volm->setLevel((double)newvol);
}


// Private slot to receive press-signals.  Assumes the position is
// relative to the main widget, not this one.
//
void pDisplay::childPressed(int xp, int yp)
{
  QPoint posn;

  posn.setX(xp);
  posn.setY(yp);
//  posn -= df_p;

  childPressed(posn);
}


// Overloaded version that does not perform position readjustment
// relative to the main widget.
//
void pDisplay::childPressed(QPoint pos)
{
  if(isInBox( pos, track_p, track_s ))
	{
	emit trackPressed( mapToGlobal(pos) );
	}
  else if(isInBox( pos, trackname_p, trackname_s ))
	{
	emit tracknamePressed( mapToGlobal(pos) );
	}
  else if(isInBox( pos, title_p, title_s ))
	{
	emit titlePressed( mapToGlobal(pos) );
	}
  else if(isInBox( pos, time_p, time_s ))
	{
	emit timePressed( mapToGlobal(pos) );
	}
  else if(isInBox( pos, modeicon_p, modeicon_s ))
	{
	emit modePressed( mapToGlobal(pos) );
	}
  else 
	{
	emit pressed( mapToGlobal(pos) );
	}
}


// Private slot to receive release-signals.  Assumes the position is
// relative to the main widget, not this one.
//
void pDisplay::childReleased(int xp, int yp)
{
  QPoint posn;

  posn.setX(xp);
  posn.setY(yp);
//  posn -= df_p;

  childReleased(posn);
}


// Overloaded version that does not perform position readjustment
// relative to the main widget.
//
void pDisplay::childReleased(QPoint pos)
{
  if(isInBox( pos, track_p, track_s ))
	{
	emit trackReleased();
	}
  else if(isInBox( pos, trackname_p, trackname_s ))
	{
	emit tracknameReleased();
	}
  else if(isInBox( pos, title_p, title_s ))
	{
	emit titleReleased();
	}
  else if(isInBox( pos, time_p, time_s ))
	{
	emit timeReleased();
	}
  else if(isInBox( pos, modeicon_p, modeicon_s ))
	{
	emit modeReleased();
	}
  else 
	{
	emit released();
	}
}


//||||||||||||||||||||||||||||||||
//
// End
//
